declare var ajv: {
  (options?: ajv.Options): ajv.Ajv;
  new(options?: ajv.Options): ajv.Ajv;
  ValidationError: typeof AjvErrors.ValidationError;
  MissingRefError: typeof AjvErrors.MissingRefError;
  $dataMetaSchema: object;
}

declare namespace AjvErrors {
  class ValidationError extends Error {
    constructor(errors: Array<ajv.ErrorObject>);

    message: string;
    errors: Array<ajv.ErrorObject>;
    ajv: true;
    validation: true;
  }

  class MissingRefError extends Error {
    constructor(baseId: string, ref: string, message?: string);
    static message: (baseId: string, ref: string) => string;

    message: string;
    missingRef: string;
    missingSchema: string;
  }
}

declare namespace ajv {
  type ValidationError = AjvErrors.ValidationError;

  type MissingRefError = AjvErrors.MissingRefError;

  interface Ajv {
    /**
    * Validate data using schema
    * Schema will be compiled and cached (using serialized JSON as key, [fast-json-stable-stringify](https://github.com/epoberezkin/fast-json-stable-stringify) is used to serialize by default).
    * @param  {string|object|Boolean} schemaKeyRef key, ref or schema object
    * @param  {Any} data to be validated
    * @return {Boolean} validation result. Errors from the last validation will be available in `ajv.errors` (and also in compiled schema: `schema.errors`).
    */
    validate(schemaKeyRef: object | string | boolean, data: any): boolean | PromiseLike<any>;
    /**
    * Create validating function for passed schema.
    * @param  {object|Boolean} schema schema object
    * @return {Function} validating function
    */
    compile(schema: object | boolean): ValidateFunction;
    /**
    * Creates validating function for passed schema with asynchronous loading of missing schemas.
    * `loadSchema` option should be a function that accepts schema uri and node-style callback.
    * @this  Ajv
    * @param {object|Boolean} schema schema object
    * @param {Boolean} meta optional true to compile meta-schema; this parameter can be skipped
    * @param {Function} callback optional node-style callback, it is always called with 2 parameters: error (or null) and validating function.
    * @return {PromiseLike<ValidateFunction>} validating function
    */
    compileAsync(schema: object | boolean, meta?: Boolean, callback?: (err: Error, validate: ValidateFunction) => any): PromiseLike<ValidateFunction>;
    /**
    * Adds schema to the instance.
    * @param {object|Array} schema schema or array of schemas. If array is passed, `key` and other parameters will be ignored.
    * @param {string} key Optional schema key. Can be passed to `validate` method instead of schema object or id/ref. One schema per instance can have empty `id` and `key`.
    * @return {Ajv} this for method chaining
    */
    addSchema(schema: Array<object> | object, key?: string): Ajv;
    /**
    * Add schema that will be used to validate other schemas
    * options in META_IGNORE_OPTIONS are alway set to false
    * @param {object} schema schema object
    * @param {string} key optional schema key
    * @return {Ajv} this for method chaining
    */
    addMetaSchema(schema: object, key?: string): Ajv;
    /**
    * Validate schema
    * @param {object|Boolean} schema schema to validate
    * @return {Boolean} true if schema is valid
    */
    validateSchema(schema: object | boolean): boolean;
    /**
    * Get compiled schema from the instance by `key` or `ref`.
    * @param  {string} keyRef `key` that was passed to `addSchema` or full schema reference (`schema.id` or resolved id).
    * @return {Function} schema validating function (with property `schema`). Returns undefined if keyRef can't be resolved to an existing schema.
    */
    getSchema(keyRef: string): ValidateFunction | undefined;
    /**
    * Remove cached schema(s).
    * If no parameter is passed all schemas but meta-schemas are removed.
    * If RegExp is passed all schemas with key/id matching pattern but meta-schemas are removed.
    * Even if schema is referenced by other schemas it still can be removed as other schemas have local references.
    * @param  {string|object|RegExp|Boolean} schemaKeyRef key, ref, pattern to match key/ref or schema object
    * @return {Ajv} this for method chaining
    */
    removeSchema(schemaKeyRef?: object | string | RegExp | boolean): Ajv;
    /**
    * Add custom format
    * @param {string} name format name
    * @param {string|RegExp|Function} format string is converted to RegExp; function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( function should return boolean ( func